/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.external;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.impl.XAttributeBooleanImpl;

import com.google.common.primitives.Booleans;

/**
 * @author F. Mannhardt
 * 
 */
final class XAttributeBooleanExternalImpl extends XAttributeExternalImpl implements XAttributeBoolean {

	private static final long serialVersionUID = 1L;
	
	private final boolean value;
	
	public XAttributeBooleanExternalImpl(int key, boolean value, ExternalStore store) {
		this(key, value, null, store);
	}

	public XAttributeBooleanExternalImpl(int key, boolean value, XExtension extension, ExternalStore store) {
		super(key, extension, store);
		this.value = value;
	}

	@Override
	public boolean getValue() {
		return value;
	}

	@Override
	public void setValue(boolean value) {
		throw new UnsupportedOperationException(IMMUTABLE_ERROR);
	}

	@Override
	public String toString() {
		return this.value ? "true" : "false";
	}

	/* (non-Javadoc)
	 * @see org.progressmining.xeslite.external.model.attributes.XAttributeExternalImpl#clone()
	 */
	public Object clone() {
		// Creating a 'normal' attribute since the clone is not yet saved in MapDB
		XAttributeBoolean clone = new XAttributeBooleanImpl(getKey(), value, getExtension());
		if (hasAttributes()) {
			clone.setAttributes(getAttributes());
		}
		return clone;
	}

	public boolean equals(Object obj) {
		if (obj instanceof XAttributeBoolean) { // compares types
			XAttributeBoolean other = (XAttributeBoolean) obj;
			return super.equals(other) // compares keys
					&& (value == other.getValue()); // compares values
		} else {
			return false;
		}
	}

	@Override
	public int compareTo(XAttribute other) {
		if (!(other instanceof XAttributeBoolean)) {
			throw new ClassCastException();
		}
		int result = super.compareTo(other);
		if (result != 0) {
			return result;
		}
		return Booleans.compare(value, ((XAttributeBoolean) other).getValue());
	}

}